;================================================================
;
; Linked list  - basic implementation
; Linked array - unsorted version
;
; Nmeth Gbor 2006.11.
;
;================================================================

IFNDEF LINKED_LIB

	LINKED_LIB EQU <LINKED_LIBRARY>


	LLIST_FLAG_PRIVATE_HEAP		EQU 1


;==============================
; STRUCTURES
;
LINKED_MEMBER	STRUCT

		lpNextLink		dd ?
		lpPrevLink		dd ?
		dwSize			dd ?
		lpParent		dd ?

LINKED_MEMBER	ENDS

LINKED_userData	EQU sizeof LINKED_MEMBER

LINKED_LIST	STRUCT

		hHeap			dd ?
		dFlags			dd ?
		dwLength		dd ?
		lpHead			dd ?
		lpTail			dd ?

LINKED_LIST	ENDS

LINKED_ARRAY	STRUCT

				LINKED_LIST <>
		dwElementCount	dd ?
		dwElementSize	dd ?
		dwSubArraySize	dd ?
		dwSubArrayLen	dd ?
		dwFreeSlots	dd ?
		lpFirstFreeSlot	dd ?

LINKED_ARRAY	ENDS

LINKED_SUBARRAY	STRUCT

		lpNextLink		dd ?
		lpPrevLink		dd ?

LINKED_SUBARRAY ENDS


LLIST_FOREACH	MACRO lpLList:REQ,DO_IT:REQ

LOCAL @1,@2,@3
			mov	ecx,lpLList
		mov	eax,(LINKED_LIST PTR [ecx]).lpHead
	@1:
		or	eax,eax
		jz @2
			push	(LINKED_MEMBER PTR [eax]).lpNextLink
			DO_IT
			pop	eax
			jmp @1
	@2:

ENDM

LARRAY_FOREACH	MACRO lpLArray:REQ,DO_IT:REQ

LOCAL @1,@2,@3
		push	ebx
		push	esi
		mov	esi,lpLArray
		mov	ebx,1
		mov	eax,(LINKED_LIST PTR [esi]).lpHead
	@1:
		or	eax,eax
		jz @3
		mov	edx,eax
		add	eax,sizeof LINKED_SUBARRAY
		mov	ecx,(LINKED_ARRAY PTR [esi]).dwSubArrayLen
	@2:
		DO_IT
		add	ebx,1
		add	eax,(LINKED_ARRAY PTR [esi]).dwElementSize
		cmp	ebx,(LINKED_ARRAY PTR [esi]).dwElementCount
		ja @3
		sub	ecx,1
		jnz @2

		mov	eax,(LINKED_MEMBER PTR [edx]).lpNextLink
		jmp @1
	@3:
		pop	esi
		pop	ebx

ENDM

;==============================
; METHODS

;================================================================
; lpLList LLIST_init(lpLList,hHeap,dwIniSize) - create a new linked list
;
; Paramters
;	lpLList		pointer to allocated space
;	hHeap		handle of memory heap or NULL
;	dwIniSize	initial size of list
;
; Returns
;	lpLList		pointer to LINKED_LIST object
;
;================================================================
	LLIST_init	PROTO lpLList:DWORD,hHeap:DWORD,dwIniSize:DWORD
	
;================================================================
; lpLINKED LLIST_create(hHeap,dwIniSize) - create a new linked list
;
; Paramters
;	hHeap		handle of memory heap or NULL
;	dwIniSize	size of heap at the beginning
;
; Returns
;	lpLList		pointer to LINKED_LIST object
;
; 1. Allocate space for list object, create private heap
; 2. Initialize list object
;
;================================================================
	LLIST_create	PROTO hHeap:DWORD,dwIniSize:DWORD

;================================================================
; void LLIST_deinit(lpLList) - deinitialize a linked list object
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;
; Returns
;	none
;
;================================================================
	LLIST_deinit	PROTO lpLList:DWORD

;================================================================
; void LLIST_destroy(lpLList) - destroy a linked list object
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;
; Returns
;	none
;
;================================================================
	LLIST_destroy	PROTO lpLList:DWORD

;================================================================
; lpMember LLIST_insert(lpLList,dwIndex,dwSize) - insert a linked list member
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;	dwIndex		index of new member
;			if zero, linked to the head
;			if -1, linked to the end
;	dwSize		size of new member in bytes (size of LINKED_MEMBER included!)
; Returns
;	lpMember	pointer to new list member object's user data
;			NULL when error
;
;================================================================
	LLIST_insert	PROTO lpLList:DWORD,dwIndex:DWORD,dwSize:DWORD

;================================================================
; void LLIST_remove(lpLList,dwIndex,lpMember) - remove a linked list member
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;	dwIndex		index of member
;	lpMember	pointer of member
;
; Returns
;	none
;
;================================================================
	LLIST_remove	PROTO lpLList:DWORD,dwIndex:DWORD,lpMember:DWORD

;================================================================
; mixed LLIST_retrieve(lpLList,dwIndex,lpMember) - look for a linked list member
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;	dwIndex		index of member
;	lpMember	pointer of member
;
; Returns
;	lpMember	pointer to list member object if lpMember argument was NULL
;	dwIndex		index to list member object if lpMember argument was supplied
;	-1		if not found
;
;================================================================
	LLIST_retrieve	PROTO lpLList:DWORD,dwIndex:DWORD,lpMember:DWORD

;================================================================
; void LLIST_link(lpLList,dwIndex,lpMember) - link a member into a list at given position
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;	dwIndex		linking position, desired index of the member
;	lpMember	pointer of member
;
; Returns
;	none		linking succesfull
;
;================================================================
	LLIST_link	PROTO lpLList:DWORD,dwIndex:DWORD,lpMember:DWORD

;================================================================
; lpMember LLIST_unlink(lpLList,dwIndex,lpMember) - unlink a member into a list at given position
;
; Paramters
;	lpLList		pointer to LINKED_LIST object
;	dwIndex		unlinking position, index of member
;	lpMember	pointer of member to be unlinked
;
; Returns
;	lpMember	pointer of unlinked member
;	NULL		selected member was not found
;
;================================================================
	LLIST_unlink	PROTO lpLList:DWORD,dwIndex:DWORD,lpMember:DWORD

;================================================================
; lpLArray LARRAY_init(lpLArray,deElementSize,dwSubArrayLength,hHeap) - initialize a new linked array
;
; Paramters
;	lpLArray	pointer to allocated space
;	deElementSize	size of one stored element in bytes
;	dwSubArrayLen	count of elements in one subarray
;	hHeap		handle of memory heap or NULL
;
; Returns
;	none
;
;================================================================
	LARRAY_init	PROTO lpLArray:DWORD,dwElementSize:DWORD,dwSubArrayLen:DWORD,hHeap:DWORD

;================================================================
; lpLArray LARRAY_create(dwElementSize,dwSubArrayLength) - create a new linked array
;
; Paramters
;	dwElementSize	size of one stored element in bytes
;	dwSubArrayLen	count of elements in one subarray
;	hHeap		handle of memory heap or NULL
;
; Returns
;	lpLArray	pointer to LINKED_ARRAY object
;
;================================================================
	LARRAY_create	PROTO dwElementSize:DWORD,dwSubArrayLen:DWORD,hHeap:DWORD

;================================================================
; void LARRAY_deinit(lpLArray) - deinitialize a linked array object
;
; Paramters
;	lpLArray	pointer to allocated space
;
; Returns
;	none
;
;================================================================
	LARRAY_deinit	TEXTEQU <LLIST_deinit>
	;LARRAY_deinit	PROTO :DWORD

;================================================================
; void LARRAY_destroy(lpLArray) - deistroy a linked array object
;
; Paramters
;	lpLArray	pointer to allocated space
;
; Returns
;	none
;
;================================================================
	LARRAY_destroy	TEXTEQU <LLIST_destroy>
	;LARRAY_destroy	PROTO :DWORD


;================================================================
; lpVoid LARRAY_insert(lpLArray) - get next free linked array entry
;
; Paramters
;	lpLArray	pointer to LINKED_ARRAY object
; Returns
;	lpVoid		pointer to new array element
;			NULL when error
;
;================================================================
	LARRAY_insert	PROTO lpLArray:DWORD

;================================================================
; dwMoved LARRAY_remove(lpLArray,dwIndex,lpElement) - remove a linked array element
;
; Paramters
;	lpLArray	pointer to LINKED_ARRAY object
;	dwIndex		index of element
;	lpElement	pointer to element
;
; Returns
;	dwMoved		index of moved element
;
;================================================================
	LARRAY_remove	PROTO lpLArray:DWORD,dwIndex:DWORD,lpElement:DWORD

;================================================================
; mixed LARRAY_retrieve(lpLArray,dwIndex) - look for a linked array element
;
; Paramters
;	lpLArray	pointer to LINKED_ARRAY object
;	dwIndex		index of requested member
;	lpElement	pointer of requested member
;
; Returns
;	lpElement	pointer to array element object if lpElement argument was NULL
;	dwIndex		index to array element object if lpElement argument was supplied
;	NULL		if not found
;
;================================================================
	LARRAY_retrieve	PROTO lpLArray:DWORD,dwIndex:DWORD,lpElement:DWORD


ENDIF
