;================================================================
;
; Graph library
;
; Definitions
;  - methods
;
;================================================================

	.586
	.model flat, stdcall
	option casemap :none

;==============================
;INCLUDES
	include \masm32\include\kernel32.inc
	include \masm32\include\windows.inc

	include graph.inc

include \masm32\include\masm32.inc
include \masm32\include\debug.inc

.code
GRPH_init	PROC USES esi lpGraph:DWORD,dwAvgDeg:DWORD
;================================================================
; bool GRPH_init(lpGraph,dwAvgDeg) - initialize a new graph
;
; Parameters
;	lpGraph		pointer to graph object
;	dwAvgDeg	average degree of vertexes
;
; Returns
;	FALSE		initialize was not succesful
;
;================================================================
		mov	esi,lpGraph
		ASSUME esi:PTR GRPH_GRAPH

	; Create private heap
		invoke HeapCreate,HEAP_NO_SERIALIZE,0,0
		or	eax,eax
		jz @End
		mov	[esi].hHeap,eax

	; Create set of vertices
		lea	eax,[esi].vertexSet
		invoke LLIST_init,eax,[esi].hHeap,NULL
		or	eax,eax
		jz @End
	; Create set of edges
		lea	eax,[esi].edgeSet
		invoke LLIST_init,eax,[esi].hHeap,NULL
		or	eax,eax
		jz @End
	; Set average degree of vertices
		mov	ecx,dwAvgDeg
		or	ecx,ecx
		jnz @F
			mov	ecx,2		; default degree is 2
	@@:
		mov	[esi].dwAvgDeg,ecx
		mov	eax,esi
@End:
		ASSUME esi:NOTHING
		ret
GRPH_init	ENDP

GRPH_create	PROC dwAvgDeg:DWORD
;================================================================
; lpGraph GRPH_create(dwAvgDeg) - create a new graph
;
; Parameters
;	dwAvgDeg	average degree of vertexes
;
; Returns
;	lpGraph		pointer to graph object
;	NULL		on error
;
;================================================================
		invoke GlobalAlloc,GPTR,sizeof GRPH_GRAPH
		or	eax,eax
		jz @End
		invoke GRPH_init,eax,dwAvgDeg
;PrintHex eax,'GRPH_create'
@End:
		ret
GRPH_create	ENDP

GRPH_deinit	PROC USES ebx esi lpGraph:DWORD
;================================================================
; void GRPH_deinit(lpGraph) - deinitialize a graph
;
; Parameters
;	lpGraph		pointer to graph object
;
; Returns
;	none
;
;================================================================
		mov	esi,lpGraph
		ASSUME esi:PTR GRPH_GRAPH
	; destroy set of vertices
		mov	ebx,[esi].hHeap
		cmp	ebx,[esi].vertexSet.hHeap		; is the heap of list also of the graph?
		jz @F
			lea	eax,[esi].vertexSet
			invoke LLIST_deinit,eax
	@@:
	; destroy set of edges
		cmp	ebx,[esi].edgeSet.hHeap			; is the heap of list also of the graph?
		jz @F
			lea	eax,[esi].edgeSet
			invoke LLIST_deinit,eax
	@@:
	; destroy heap
		invoke HeapDestroy,[esi].hHeap
		ASSUME esi:NOTHING
		ret
GRPH_deinit	ENDP

GRPH_destroy	PROC lpGraph:DWORD
;================================================================
; void GRPH_destroy(lpGraph) - destroy a graph
;
; Parameters
;	lpGraph		pointer to graph object
;
; Returns
;	none
;
;================================================================
		invoke GRPH_deinit,lpGraph
		invoke GlobalFree,lpGraph

		ret
GRPH_destroy	ENDP

GRPH_addVertex	PROC USES ebx esi lpGraph:DWORD,dwSize:DWORD
;================================================================
; lpVertex GRPH_addVertex(lpGraph,dwSize) - insert a vertex
;
; Parameters
;	lpGraph		pointer to graph object
;	dwSize		size of user data stored in vertex
;
; Returns
;	lpVertex	pointer to vertex object
;	NULL		on error
;
; return llistInsert(lpGraph->vertexSet,-1,sizeof VERTEX+dwSize);
;
;================================================================
		mov	ecx,dwSize
		mov	esi,lpGraph
		ASSUME esi:PTR GRPH_GRAPH
		;add	ecx,sizeof GRPH_VERTEX-sizeof LINKED_MEMBER
		;add	ecx,sizeof GRPH_VERTEX
		invoke LLIST_insert,esi,-1,ecx		; because the vertex set is the first field of the graph
		or	eax,eax
		jz @F
	; invoke LARRAY_init,eax.edgesIn,4,[esi].dwAvgDeg,[esi].hHeap
			push	DWORD PTR [esi].hHeap
		mov	ebx,eax
			push	DWORD PTR [esi].dwAvgDeg
		add	eax,GRPH_VERTEX.edgesIn
			push	DWORD PTR 4
			push	eax
			call LARRAY_init
		push	ebx
	;invoke LARRAY_init,edgesOut,4,[esi].dwAvgDeg,[esi].hHeap
			push	DWORD PTR [esi].hHeap
			add	ebx,GRPH_VERTEX.edgesOut
			push	DWORD PTR [esi].dwAvgDeg
			push	DWORD PTR 4
			push	ebx
			call LARRAY_init
		pop	eax
;PrintHex eax,'addVertex'
	@@:
		ret
		ASSUME esi:NOTHING
GRPH_addVertex	ENDP

GRPH_delVertex	PROC USES ebx edi esi lpGraph:DWORD,lpVertex:DWORD
;================================================================
;
; void GRPH_delVertex(lpGraph,lpVertex) - remove a vertex
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex	pointer to vertex object
;
; Returns
;	none
;
; foreach (lpEdge in lpVertex->edgesOut) delEdge(lpGraph,lpEdge);
; foreach (lpEdge in lpVertex->edgesIn) delEdge(lpGraph,lpEdge);
; llistRemove(lpGraph->vertexSet,0,lpVertex)
;
;================================================================
		mov	esi,lpVertex
		ASSUME esi:PTR GRPH_VERTEX

;PrintHex esi,'delVertex'

@1: ; foreach (lpEdge in lpVertex->edgesOut) delEdge(lpGraph,lpEdge)
		mov	ecx,[esi].edgesOut.dwElementCount
		mov	eax,[esi].edgesOut.lpHead
	; is array empty?
		or	ecx,ecx
		jz @2
			mov	ebx,sizeof LINKED_SUBARRAY
			or	eax,eax
			jz @2
		;jmp @12
ALIGN 4
	@11:
		mov	edi,eax
ALIGN 4
	@@:
		push	ecx
		invoke GRPH_delEdge,lpGraph,DWORD PTR [edi+ebx],GRPH_FLAG_REMOVE_IN
		pop	ecx
		add	ebx,4	;[esi].edgesOut.dwElementSize
		sub	ecx,1
		jz @2
		cmp	ebx,[esi].edgesOut.dwSubArraySize
		jc @B
		mov	eax,(LINKED_SUBARRAY PTR [edi]).lpNextLink
	@12:
		mov	ebx,sizeof LINKED_SUBARRAY
		or	eax,eax
		jnz @11

@2: ; foreach (lpEdge in lpVertex->edgesIn) delEdge(lpGraph,lpEdge)
		mov	ecx,[esi].edgesIn.dwElementCount
		mov	eax,[esi].edgesIn.lpHead
	; is array empty?
		or	ecx,ecx
		jz @3
			mov	ebx,sizeof LINKED_SUBARRAY
			or	eax,eax
			jz @3
		;jmp @22
ALIGN 4
	@21:
		mov	edi,eax
ALIGN 4
	@@:
		push	ecx
		invoke GRPH_delEdge,lpGraph,DWORD PTR [edi+ebx],GRPH_FLAG_REMOVE_OUT
		pop	ecx
		add	ebx,4	;[esi].edgesIn.dwElementSize
		sub	ecx,1
		jz @3
		cmp	ebx,[esi].edgesIn.dwSubArraySize
		jc @B
		mov	eax,(LINKED_SUBARRAY PTR [edi]).lpNextLink
	@22:
		mov	ebx,sizeof LINKED_SUBARRAY
		or	eax,eax
		jnz @21

@3: ; llistRemove(lpGraph->vertexSet,NULL,lpVertex)
		mov	eax,lpGraph
		push	lpVertex
		add	eax,GRPH_GRAPH.vertexSet
		push	NULL
		push	eax
;PrintHex esi,'delVertex'
		call LLIST_remove
@4:
		ret
		ASSUME esi:NOTHING
GRPH_delVertex	ENDP

GRPH_addEdge	PROC USES esi lpGraph:DWORD,lpVertex1:DWORD,lpVertex2:DWORD,dwSize:DWORD
;================================================================
;
; lpEdge GRPH_addEdge(lpGraph,lpVertex1,lpVertex2,dwSize) - insert an edge
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex1	pointer to start vertex object
;	lpVertex2	pointer to end vertex object
;	dwSize		size of user data stored in edge
;
; Returns
;	lpEdge		pointer to edge object
;	NULL		on error
;
;================================================================
		mov	ecx,dwSize
		mov	esi,lpGraph
		ASSUME esi:PTR GRPH_GRAPH
		;add	ecx,sizeof GRPH_EDGE-sizeof LINKED_MEMBER
		add	esi,GRPH_GRAPH.edgeSet
	; insert edge
		invoke LLIST_insert,esi,-1,ecx
		or	eax,eax
		jz @End
	; store lpVertex1 in edge struct
			mov	ecx,lpVertex1
		mov	esi,eax
			push	(GRPH_VERTEX PTR [ecx]).edgesOut.dwElementCount
			mov	(GRPH_EDGE PTR [eax]).lpVertexOut,ecx
			add	ecx,GRPH_VERTEX.edgesOut
			pop	(GRPH_EDGE PTR [eax]).dwIndexOut
			invoke LARRAY_insert,ecx
		or	eax,eax
		jz @End
	; store lpVertex2 in edge struct
			mov	ecx,lpVertex2
		mov	[eax],esi				; store edge's pointer in 'edgesOut' array of lpVertex1
			push	(GRPH_VERTEX PTR [ecx]).edgesIn.dwElementCount
			mov	(GRPH_EDGE PTR [esi]).lpVertexIn,ecx
			add	ecx,GRPH_VERTEX.edgesIn
			pop	(GRPH_EDGE PTR [esi]).dwIndexIn
			invoke LARRAY_insert,ecx
		or	eax,eax
		jz @End
		mov	[eax],esi				; store edge's pointer in 'edgesIn' array of lpVertex2
	; Return lpEdge
		mov	eax,esi
	@End:
		ret
		ASSUME esi:NOTHING
GRPH_addEdge	ENDP

GRPH_delEdge	PROC USES ebx esi lpGraph:DWORD,lpEdge:DWORD,dFlag:DWORD
;================================================================
;
; void GRPH_delEdge(lpGraph,lpEdge) - remove an edge
;
; Parameters
;	lpGraph		pointer to graph object
;	lpEdge		pointer to edge object
;	dFlag		0-1. bit: remove type
;				01 - remove from lpVertex1
;				10 - remove from lpVertex2
;				11 - remove from both
;
; Returns
;	FALSE		remove was not succesfull
;
;================================================================
		mov	esi,lpGraph
		mov	ebx,lpEdge
		add	esi,GRPH_GRAPH.edgeSet
	; get the edge
		invoke LLIST_retrieve,esi,NULL,ebx
		cmp	eax,-1
		jz @End

	; remove edge at the staring point?
		test	dFlag,GRPH_FLAG_REMOVE_OUT
		jz @F
	; remove edge from 'edgesOut' array of lpVertex1
		mov	eax,(GRPH_EDGE PTR [ebx]).lpVertexOut
		push	NULL
		add	eax,GRPH_VERTEX.edgesOut
		push	DWORD PTR (GRPH_EDGE PTR [ebx]).dwIndexOut
		push	eax
		call LARRAY_remove
		;invoke LARRAY_remove,eax,(GRPH_EDGE PTR [ebx]).dwIndexOut,NULL

	@@: ; remove edge at the ending point?
		test dFlag,GRPH_FLAG_REMOVE_IN
		jz @F
	; remove edge from 'edgesIn' array of lpVertex2
		mov	eax,(GRPH_EDGE PTR [ebx]).lpVertexIn
		push	NULL
		add	eax,GRPH_VERTEX.edgesIn
		push	DWORD PTR (GRPH_EDGE PTR [ebx]).dwIndexIn
		push	eax
		call LARRAY_remove
	@@:
	; remove the edge
		invoke LLIST_remove,esi,NULL,lpEdge

	; Return value
	@End:
		ret
GRPH_delEdge	ENDP


GRPH_isAdjacent	PROC USES ebx edi esi lpVertex1:DWORD,lpVertex2:DWORD,bDirected:DWORD

		push	ebp

		mov	esi,lpVertex1
		mov	edx,lpVertex2
	@checkEdgesOut:
		mov	edi,(GRPH_VERTEX PTR [esi]).edgesOut.lpHead		; get first subarray
		mov	ebx,(GRPH_VERTEX PTR [esi]).edgesOut.dwElementCount

		@1:
			or	edi,edi		; no subarray(=no edges) yet?
			jz @checkEdgesIn
			mov	ebp,edi
			mov	ecx,(GRPH_VERTEX PTR [esi]).edgesOut.dwSubArrayLen
			add	edi,sizeof LINKED_SUBARRAY
		@11:
			mov	eax,[edi]	; get edge
			cmp	(GRPH_EDGE PTR [eax]).lpVertexIn,edx		; edge links to lpVertex2? 
			jz @EdgeFound
			add	edi,(GRPH_VERTEX PTR [esi]).edgesOut.dwElementSize
			sub	ebx,1
			jz @checkEdgesIn
			sub	ecx,1
			jnz @11
			mov	edi,(LINKED_MEMBER PTR [ebp]).lpNextLink	; next subarray
			jmp @1

	@checkEdgesIn:
		mov	edi,(GRPH_VERTEX PTR [esi]).edgesIn.lpHead
		mov	ebx,(GRPH_VERTEX PTR [esi]).edgesIn.dwElementCount

		@2:
			or	edi,edi		; no subarray(=no edges) yet?
			jz @EdgeNotFound
			mov	ebp,edi
			mov	ecx,(GRPH_VERTEX PTR [esi]).edgesIn.dwSubArrayLen
			add	edi,sizeof LINKED_SUBARRAY
		@21:
			mov	eax,[edi]	; get edge
			cmp	(GRPH_EDGE PTR [eax]).lpVertexOut,edx		; edge links to lpVertex2? 
			jz @EdgeFound
			add	edi,(GRPH_VERTEX PTR [esi]).edgesIn.dwElementSize
			sub	ebx,1
			jz @EdgeNotFound
			sub	ecx,1
			jnz @21
			mov	edi,(LINKED_MEMBER PTR [ebp]).lpNextLink	; next subarray
			jmp @2
	@EdgeNotFound:
		xor	eax,eax		; return FALSE
	@EdgeFound:
		pop	ebp

		ret
GRPH_isAdjacent	ENDP
end