;================================================================
;
; GRAPH LIBRARY
;
; Declarations
;  - data structures
;  - methods
;
;================================================================

IFNDEF GRPH_LIBRARY

	GRPH_LIBRARY	EQU <GRAPH_LIBRARY>

	include linked.inc
	includelib linked.lib

	GRPH_FLAG_REMOVE_OUT	EQU 01b
	GRPH_FLAG_REMOVE_IN	EQU 10b

	LPGRPH	TYPEDEF PTR GRPH_GRPAH


;==============================
; STRUCTURES
;
GRPH_GRAPH	STRUCT

		vertexSet	LINKED_LIST <>
		edgeSet		LINKED_LIST <>
		dwAvgDeg	dd ?
		hHeap		dd ?

GRPH_GRAPH	ENDS

;GRPH_EDGEINFO	STRUCT
;
;		lpEdge		dd ?
;		lpVertex	dd ?
;
;GRPH_EDGEINFO	ENDS

GRPH_VERTEX	STRUCT

				LINKED_MEMBER <>
		edgesOut	LINKED_ARRAY <>
		edgesIn		LINKED_ARRAY <>
		dwColor		dd ?
		dwFlag		dd ?

GRPH_VERTEX	ENDS

GRPH_EDGE	STRUCT

				LINKED_MEMBER <>
		lpVertexOut	dd ?
		dwIndexOut	dd ?
		lpVertexIn	dd ?
		dwIndexIn	dd ?
		dwWeight	REAL4 ?
		dwColor		dd ?

GRPH_EDGE	ENDS

;==============================
; METHODS

;================================================================
; void GRPH_init(lpGraph,dwAvgDeg) - initialize a new graph
;
; Parameters
;	lpGraph		pointer to graph object
;	dwAvgDeg	average degree of vertexes
;
; Returns
;	none
;
;================================================================
	GRPH_init	PROTO lpGraph:DWORD,dwAvgDeg:DWORD

;================================================================
; lpGraph GRPH_create(dwAvgDeg) - create a new graph
;
; Parameters
;	dwAvgDeg	average degree of vertexes
;
; Returns
;	lpGraph		pointer to graph object
;	NULL		on error
;
;================================================================
	GRPH_create	PROTO dwAvgDeg:DWORD

;================================================================
; void GRPH_deinit(lpGraph) - deinitialize a graph
;
; Parameters
;	lpGraph		pointer to graph object
;
; Returns
;	none
;
;================================================================
	GRPH_deinit	PROTO lpGraph:DWORD

;================================================================
; void GRPH_destroy(lpGraph) - destroy a graph
;
; Parameters
;	lpGraph		pointer to graph object
;
; Returns
;	none
;
;================================================================
	GRPH_destroy	PROTO lpGraph:DWORD

;================================================================
; lpVertex GRPH_addVertex(lpGraph) - insert a vertex
;
; Parameters
;	lpGraph		pointer to graph object
;	dwSize		size of user data stored in vertex
;
; Returns
;	lpVertex	pointer to vertex object
;	NULL		on error
;
;================================================================
	GRPH_addVertex	PROTO lpGraph:DWORD,dwSize:DWORD

;================================================================
;
; void GRPH_delVertex(lpGraph,lpVertex) - remove a vertex
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex	pointer to vertex object
;
; Returns
;	none
;
;================================================================
	GRPH_delVertex	PROTO lpGraph:DWORD,lpVertex:DWORD

;================================================================
;
; lpEdge GRPH_addEdge(lpGraph,lpVertex1,lpVertex2,dwSize) - insert an edge
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex1	pointer to start vertex object
;	lpVertex2	pointer to end vertex object
;	dwSize		size of user data stored in edge
;
; Returns
;	lpEdge		pointer to edge object
;	NULL		on error
;
;================================================================
	GRPH_addEdge	PROTO lpGraph:DWORD,lpVertex1:DWORD,lpVertex2:DWORD,dwSize:DWORD

;================================================================
;
; void GRPH_delEdge(lpGraph,lpEdge) - remove an edge
;
; Parameters
;	lpGraph		pointer to graph object
;	lpEdge		pointer to edge object
;	dFlag		0-1. bit: remove type
;				01 - remove from lpVertex1
;				10 - remove from lpVertex2
;				11 - remove from both
;
; Returns
;	FALSE		remove was not succesfull
;
;================================================================
	GRPH_delEdge	PROTO lpGraph:DWORD,lpEdge:DWORD,dFlag:DWORD



;================================================================
;
; lpEdge GRPH_isAdjacent(lpVertex1,lpVertex1,bDirected) - checks whether the 2 vertices are adjacent
;
; Parameters
;	lpVertex1	pointer to base vertex
;	lpVertex2	pointer to vertex to be checked
;	bDirected	flag for directed graph
;			TRUE - graph is directed (only outgoing edges are checked)
;			FALSE - graph is indirected (both, outgoing and incoming edges are checked)
; Returns
;	lpEdge		pointer of the edge between the vertices
;	NULL		vertices are not adjacent
;
;================================================================
	GRPH_isAdjacent	PROTO lpVertex1:DWORD,lpVertex2:DWORD,bDirected:DWORD

;================================================================
;
; void GRPH_DFS(lpGraph,lpVertex) - Depth First Search
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex	pointer to staring edge
;
; Returns
;	none
;
;================================================================
	GRPH_DFS	PROTO lpGraph:DWORD,lpVertex:DWORD

;================================================================
;
; void GRPH_DFS(lpGraph,lpVertex) - Breadth First Search
;
; Parameters
;	lpGraph		pointer to graph object
;	lpVertex	pointer to staring edge
;
; Returns
;	none
;
;================================================================
	GRPH_BFS	PROTO lpGraph:DWORD,lpVertex:DWORD

ENDIF